import { CSSDirective } from "@microsoft/fast-element";
import type { DesignTokenValue, StaticDesignTokenValue } from "./interfaces";
/**
 * Describes a DesignToken instance.
 * @alpha
 */
export interface DesignToken<T extends string | number | boolean | BigInteger | null | Array<any> | symbol | {
    createCSS?(): string;
}> extends CSSDirective {
    readonly name: string;
    /**
     * The {@link (DesignToken:interface)} formatted as a CSS custom property if the token is
     * configured to write a CSS custom property, otherwise empty string;
     */
    readonly cssCustomProperty: string;
    /**
     * Get the token value for an element.
     * @param element - The element to get the value for
     * @returns - The value set for the element, or the value set for the nearest element ancestor.
     */
    getValueFor(element: HTMLElement): StaticDesignTokenValue<T>;
    /**
     * Sets the token to a value for an element.
     * @param element - The element to set the value for.
     * @param value - The value.
     */
    setValueFor(element: HTMLElement, value: DesignTokenValue<T> | DesignToken<T>): void;
    /**
     * Removes a value set for an element.
     * @param element - The element to remove the value from
     */
    deleteValueFor(element: HTMLElement): this;
    /**
     * Associates a default value to the token
     */
    withDefault(value: DesignTokenValue<T> | DesignToken<T>): this;
}
declare function create<T extends Function>(name: string): never;
declare function create<T extends undefined | void>(name: string): never;
declare function create<T>(name: string): DesignToken<T>;
/**
 * Factory object for creating {@link (DesignToken:interface)} instances.
 * @alpha
 */
export declare const DesignToken: Readonly<{
    create: typeof create;
}>;
export {};
