import { css, DOM, FASTElement } from "@microsoft/fast-element";
/**
 * Caching mechanism for CSS custom properties
 */
class CustomPropertyManagerImpl {
    /**
     * {@inheritdoc CustomPropertyManager.get}
     */
    getElementStyles(key, value) {
        let keyCache = CustomPropertyManagerImpl.cache.get(key.cssCustomProperty);
        if (!keyCache) {
            keyCache = new Map();
            CustomPropertyManagerImpl.cache.set(key.cssCustomProperty, keyCache);
        }
        let v = keyCache.get(value);
        if (!v) {
            v = this.createElementStyles(key, value);
            keyCache.set(value, v);
        }
        return v;
    }
    /**
     * Creates an ElementStyles with the key/value CSS custom property
     * on the host
     */
    createElementStyles(token, value) {
        return css `:host{${token.cssCustomProperty}:${value};}`;
    }
    addTo(element, token, value) {
        if (isFastElement(element)) {
            element.$fastController.addStyles(this.getElementStyles(token, value));
        }
        else {
            DOM.queueUpdate(() => element.style.setProperty(token.cssCustomProperty, value));
        }
    }
    removeFrom(element, token, value) {
        if (isFastElement(element)) {
            element.$fastController.removeStyles(this.getElementStyles(token, value));
        }
        else if (element.style.getPropertyValue(token.cssCustomProperty) === value) {
            DOM.queueUpdate(() => element.style.removeProperty(token.cssCustomProperty));
        }
    }
}
CustomPropertyManagerImpl.cache = new Map();
function isFastElement(element) {
    return element instanceof FASTElement;
}
/**
 * @internal
 */
export const CustomPropertyManager = new CustomPropertyManagerImpl();
